@@ -44,6 +44,18 @@ export default class Tag extends Component<TagProps> {
         this.state = { hovered: false };
     }
 
+    componentDidMount(): void {
+        const {treeNodeSelection} = this.props;
+        const input = (
+            treeNodeSelection.getRef() as React.RefObject<HTMLInputElement>
+        ).current as HTMLInputElement;
+        if (input) {
+            const value = input.value === "" ? input.placeholder : input.value;
+            const width = this.calculateTextWidth(value);
+            input.style.width = width + "px";
+        }
+    }
+
     private addAdditionalClasses(invalid: boolean): boolean {
         const { currentTag, treeNodeSelection } = this.props;
         return (
@@ -283,6 +295,7 @@ export default class Tag extends Component<TagProps> {
             index,
             treeNodeSelection,
             currentTag,
+            placeholder,
             checkIfDuplicate,
             inputKeyDown,
             inputKeyUp,
@@ -343,7 +356,7 @@ export default class Tag extends Component<TagProps> {
                                 (
                                     treeNodeSelection.getFocussedNodeName() === ""
                                         && treeNodeSelection.getFocussedLevel() == 0
-                                        ? "Add new tag..." : ""
+                                        ? placeholder : ""
                                 )
                             }
                             value={displayText}
